/*
 * bigecho.c
 * Copyright (C) 2001 Kentarou Fukuchi <fukuchi@users.sourceforge.net>
 *
 * The most part of code is derived from GNU shellutils package.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <termios.h>
#include <sys/ioctl.h>

/* The name this program was run with. */
char *program_name;

#ifndef STDOUT_FILENO
#define STDOUT_FILENO 1
#endif

extern unsigned char fontdata_8x8[];

char *
concat(int words, char **word)
{
	int total_length = 0;
	int i;
	char *str;

	for(i=0; i<words; i++) {
		total_length += strlen(word[i]);
		if(i+1 < words)
			total_length++;
	}
	
	str = (char *)malloc(total_length+1);
	if(str == NULL) return NULL;

	str[0] = '\0';
	for(i=0; i<words; i++)  {
		strcat(str, word[i]);
		if(i+1 < words)
			strcat(str, " ");
	}

	return str;
}

int
bigecho(char *str, int display_return)
{
	int width = 79;
	int line_bigchars;
	int str_length;
	int lastline = 0;
	int x, y, r, i;
	int rows;
	int chars;
	unsigned char bit, pattern;

#ifdef TIOCGWINSZ
	{
		struct winsize ws;
		if(ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws) != 1 && ws.ws_col != 0)
			width = ws.ws_col - 1;
	}
#endif

	line_bigchars = width / 8;
	if(line_bigchars < 1) return -1;

	str_length = strlen(str);
	rows = (str_length + line_bigchars - 1) / line_bigchars;

	for(r=0; r<rows; r++) {
		if(str_length > line_bigchars) {
			chars = line_bigchars;
		} else {
			chars = str_length;
			lastline = 1;
		}
		for(y=0; y<8; y++) {
			for(x=0; x<chars; x++) {
				pattern = fontdata_8x8[((unsigned char)str[x])*8+y];
				bit = 0x80;
				for(i=0; i<8; i++) {
					if(bit & pattern) {
						putchar('#');
					} else {
						putchar(' ');
					}
					bit = bit>>1;
				}
			}
			if(y != 7 || !lastline) {
				putchar('\n');
			}
		}
		str += chars;
		str_length -= chars;
	}
	if(display_return) {
		putchar('\n');
	}
	return 0;
}

/* Print the words in LIST to standard output.  If the first word is
   `-n', then don't print a trailing newline. */

int
main (int argc, char **argv)
{
  int display_return = 1;
  char *string;

  program_name = argv[0];

  --argc;
  ++argv;

  while (argc > 0 && *argv[0] == '-')
    {
      register char *temp;
      register int i;

      /* If it appears that we are handling options, then make sure that
	 all of the options specified are actually valid.  Otherwise, the
	 string should just be echoed. */
      temp = argv[0] + 1;

      for (i = 0; temp[i]; i++)
	{
	  if (strrchr ("n", temp[i]) == 0)
	    goto just_echo;
	}

      if (!*temp)
	goto just_echo;

      /* All of the options in TEMP are valid options to ECHO.
	 Handle them. */
      while (*temp)
	{
	  if (*temp == 'n')
	    display_return = 0;
	  else
	    goto just_echo;

	  temp++;
	}
      argc--;
      argv++;
    }

just_echo:

  if (argc > 0)
    {
	  string = concat(argc, argv);
	  if(string != NULL)
	  	if(bigecho(string, display_return)) {
			exit(1);
		}
    }
  exit (0);
}
